@@ -31,6 +31,7 @@ struct RowGroupPruningTest {
     expected_errors: Option<usize>,
     expected_row_group_matched_by_statistics: Option<usize>,
     expected_row_group_pruned_by_statistics: Option<usize>,
+    expected_files_pruned_by_statistics: Option<usize>,
     expected_row_group_matched_by_bloom_filter: Option<usize>,
     expected_row_group_pruned_by_bloom_filter: Option<usize>,
     expected_results: usize,
@@ -44,6 +45,7 @@ impl RowGroupPruningTest {
             expected_errors: None,
             expected_row_group_matched_by_statistics: None,
             expected_row_group_pruned_by_statistics: None,
+            expected_files_pruned_by_statistics: None,
             expected_row_group_matched_by_bloom_filter: None,
             expected_row_group_pruned_by_bloom_filter: None,
             expected_results: 0,
@@ -80,6 +82,11 @@ impl RowGroupPruningTest {
         self
     }
 
+    fn with_pruned_files(mut self, pruned_files: Option<usize>) -> Self {
+        self.expected_files_pruned_by_statistics = pruned_files;
+        self
+    }
+
     // Set the expected matched row groups by bloom filter
     fn with_matched_by_bloom_filter(mut self, matched_by_bf: Option<usize>) -> Self {
         self.expected_row_group_matched_by_bloom_filter = matched_by_bf;
@@ -121,6 +128,11 @@ impl RowGroupPruningTest {
             self.expected_row_group_pruned_by_statistics,
             "mismatched row_groups_pruned_statistics",
         );
+        assert_eq!(
+            output.files_ranges_pruned_statistics(),
+            self.expected_files_pruned_by_statistics,
+            "mismatched files_ranges_pruned_statistics",
+        );
         assert_eq!(
             output.row_groups_matched_bloom_filter(),
             self.expected_row_group_matched_by_bloom_filter,
@@ -148,6 +160,7 @@ async fn prune_timestamps_nanos() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(10)
@@ -165,6 +178,7 @@ async fn prune_timestamps_micros() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(10)
@@ -182,6 +196,7 @@ async fn prune_timestamps_millis() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(10)
@@ -199,6 +214,7 @@ async fn prune_timestamps_seconds() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(10)
@@ -214,6 +230,7 @@ async fn prune_date32() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(3))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(1)
@@ -256,6 +273,7 @@ async fn prune_disabled() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(10)
@@ -301,6 +319,7 @@ macro_rules! int_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(3))
                     .with_pruned_by_stats(Some(1))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(0))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(11)
@@ -315,6 +334,7 @@ macro_rules! int_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(3))
                     .with_pruned_by_stats(Some(1))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(0))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(11)
@@ -330,6 +350,7 @@ macro_rules! int_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(1))
                     .with_pruned_by_stats(Some(3))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(1))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(1)
@@ -344,6 +365,7 @@ macro_rules! int_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(1))
                     .with_pruned_by_stats(Some(3))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(1))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(1)
@@ -359,6 +381,7 @@ macro_rules! int_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(0))
                     .with_pruned_by_stats(Some(0))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(0))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(3)
@@ -374,6 +397,7 @@ macro_rules! int_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(0))
                     .with_pruned_by_stats(Some(0))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(0))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(2)
@@ -389,6 +413,7 @@ macro_rules! int_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(0))
                     .with_pruned_by_stats(Some(0))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(0))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(9)
@@ -405,6 +430,7 @@ macro_rules! int_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(1))
                     .with_pruned_by_stats(Some(3))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(1))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(1)
@@ -421,7 +447,8 @@ macro_rules! int_tests {
                     .with_query(&format!("SELECT * FROM t where i{} in (100)", $bits))
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(0))
-                    .with_pruned_by_stats(Some(4))
+                    .with_pruned_by_stats(Some(0))
+                    .with_pruned_files(Some(1))
                     .with_matched_by_bloom_filter(Some(0))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(0)
@@ -438,6 +465,7 @@ macro_rules! int_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(4))
                     .with_pruned_by_stats(Some(0))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(4))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(19)
@@ -467,6 +495,7 @@ macro_rules! uint_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(3))
                     .with_pruned_by_stats(Some(1))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(0))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(11)
@@ -482,6 +511,7 @@ macro_rules! uint_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(1))
                     .with_pruned_by_stats(Some(3))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(1))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(1)
@@ -496,6 +526,7 @@ macro_rules! uint_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(1))
                     .with_pruned_by_stats(Some(3))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(1))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(1)
@@ -511,6 +542,7 @@ macro_rules! uint_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(0))
                     .with_pruned_by_stats(Some(0))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(0))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(2)
@@ -526,6 +558,7 @@ macro_rules! uint_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(0))
                     .with_pruned_by_stats(Some(0))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(0))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(2)
@@ -542,6 +575,7 @@ macro_rules! uint_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(1))
                     .with_pruned_by_stats(Some(3))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(1))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(1)
@@ -559,6 +593,7 @@ macro_rules! uint_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(0))
                     .with_pruned_by_stats(Some(4))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(0))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(0)
@@ -575,6 +610,7 @@ macro_rules! uint_tests {
                     .with_expected_errors(Some(0))
                     .with_matched_by_stats(Some(4))
                     .with_pruned_by_stats(Some(0))
+                    .with_pruned_files(Some(0))
                     .with_matched_by_bloom_filter(Some(4))
                     .with_pruned_by_bloom_filter(Some(0))
                     .with_expected_rows(19)
@@ -604,6 +640,7 @@ async fn prune_int32_eq_large_in_list() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(1))
         .with_expected_rows(0)
@@ -626,6 +663,7 @@ async fn prune_uint32_eq_large_in_list() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(1))
         .with_expected_rows(0)
@@ -641,6 +679,7 @@ async fn prune_f64_lt() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(11)
@@ -652,6 +691,7 @@ async fn prune_f64_lt() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(11)
@@ -669,6 +709,7 @@ async fn prune_f64_scalar_fun_and_gt() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(2))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(1)
@@ -685,6 +726,7 @@ async fn prune_f64_scalar_fun() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(0))
         .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(1)
@@ -701,6 +743,7 @@ async fn prune_f64_complex_expr() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(0))
         .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(9)
@@ -717,6 +760,7 @@ async fn prune_f64_complex_expr_subtract() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(0))
         .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(9)
@@ -735,6 +779,7 @@ async fn prune_decimal_lt() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(6)
@@ -746,6 +791,7 @@ async fn prune_decimal_lt() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(8)
@@ -757,6 +803,7 @@ async fn prune_decimal_lt() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(6)
@@ -768,6 +815,7 @@ async fn prune_decimal_lt() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(8)
@@ -786,6 +834,7 @@ async fn prune_decimal_eq() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(2))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(2)
@@ -797,6 +846,7 @@ async fn prune_decimal_eq() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(2))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(2)
@@ -809,6 +859,7 @@ async fn prune_decimal_eq() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(2))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(2)
@@ -820,6 +871,7 @@ async fn prune_decimal_eq() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(2))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(2)
@@ -839,6 +891,7 @@ async fn prune_decimal_in_list() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(5)
@@ -850,6 +903,7 @@ async fn prune_decimal_in_list() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(6)
@@ -861,6 +915,7 @@ async fn prune_decimal_in_list() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(5)
@@ -872,6 +927,7 @@ async fn prune_decimal_in_list() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(6)
@@ -885,6 +941,7 @@ async fn prune_decimal_in_list() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(1))
         .with_pruned_by_bloom_filter(Some(2))
         .with_expected_rows(1)
@@ -898,6 +955,7 @@ async fn prune_decimal_in_list() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(1))
         .with_pruned_by_bloom_filter(Some(2))
         .with_expected_rows(1)
@@ -911,6 +969,7 @@ async fn prune_decimal_in_list() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(1))
         .with_pruned_by_bloom_filter(Some(2))
         .with_expected_rows(1)
@@ -929,6 +988,7 @@ async fn prune_string_eq_match() {
         // false positive on 'all backends' batch: 'backend five' < 'backend one' < 'backend three'
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(1))
         .with_pruned_by_bloom_filter(Some(1))
         .with_expected_rows(1)
@@ -947,6 +1007,7 @@ async fn prune_string_eq_no_match() {
         // false positive on 'all backends' batch: 'backend five' < 'backend one' < 'backend three'
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(2))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(1))
         .with_expected_rows(0)
@@ -963,6 +1024,7 @@ async fn prune_string_eq_no_match() {
         // false positive on 'mixed' batch: 'backend one' < 'frontend nine' < 'frontend six'
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(2))
         .with_expected_rows(0)
@@ -980,6 +1042,7 @@ async fn prune_string_neq() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(3))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(14)
@@ -998,6 +1061,7 @@ async fn prune_string_lt() {
         // matches 'all backends' only
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(2))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(3)
@@ -1012,6 +1076,7 @@ async fn prune_string_lt() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         // all backends from 'mixed' and 'all backends'
@@ -1031,6 +1096,7 @@ async fn prune_binary_eq_match() {
         // false positive on 'all backends' batch: 'backend five' < 'backend one' < 'backend three'
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(1))
         .with_pruned_by_bloom_filter(Some(1))
         .with_expected_rows(1)
@@ -1049,6 +1115,7 @@ async fn prune_binary_eq_no_match() {
         // false positive on 'all backends' batch: 'backend five' < 'backend one' < 'backend three'
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(2))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(1))
         .with_expected_rows(0)
@@ -1065,6 +1132,7 @@ async fn prune_binary_eq_no_match() {
         // false positive on 'mixed' batch: 'backend one' < 'frontend nine' < 'frontend six'
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(2))
         .with_expected_rows(0)
@@ -1082,6 +1150,7 @@ async fn prune_binary_neq() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(3))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(14)
@@ -1100,6 +1169,7 @@ async fn prune_binary_lt() {
         // matches 'all backends' only
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(2))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(3)
@@ -1114,6 +1184,7 @@ async fn prune_binary_lt() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         // all backends from 'mixed' and 'all backends'
@@ -1133,6 +1204,7 @@ async fn prune_fixedsizebinary_eq_match() {
         // false positive on 'all frontends' batch: 'fe1' < 'fe6' < 'fe7'
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(1))
         .with_pruned_by_bloom_filter(Some(1))
         .with_expected_rows(1)
@@ -1148,6 +1220,7 @@ async fn prune_fixedsizebinary_eq_match() {
         // false positive on 'all frontends' batch: 'fe1' < 'fe6' < 'fe7'
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(1))
         .with_pruned_by_bloom_filter(Some(1))
         .with_expected_rows(1)
@@ -1166,6 +1239,7 @@ async fn prune_fixedsizebinary_eq_no_match() {
         // false positive on 'mixed' batch: 'be1' < 'be9' < 'fe4'
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(2))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(1))
         .with_expected_rows(0)
@@ -1183,6 +1257,7 @@ async fn prune_fixedsizebinary_neq() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(3))
         .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(3))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(14)
@@ -1201,6 +1276,7 @@ async fn prune_fixedsizebinary_lt() {
         // matches 'all backends' only
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(2))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(2)
@@ -1215,6 +1291,7 @@ async fn prune_fixedsizebinary_lt() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
         // all backends from 'mixed' and 'all backends'
@@ -1235,6 +1312,7 @@ async fn prune_periods_in_column_names() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(2))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(7)
@@ -1246,6 +1324,7 @@ async fn prune_periods_in_column_names() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(2))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(1))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(5)
@@ -1257,6 +1336,7 @@ async fn prune_periods_in_column_names() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(2))
+        .with_pruned_files(Some(0))
         .with_matched_by_bloom_filter(Some(1))
         .with_pruned_by_bloom_filter(Some(0))
         .with_expected_rows(2)
@@ -1277,6 +1357,7 @@ async fn test_row_group_with_null_values() {
         .with_query("SELECT * FROM t WHERE \"i8\" <= 5")
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_pruned_by_stats(Some(2))
         .with_expected_rows(5)
         .with_matched_by_bloom_filter(Some(0))
@@ -1290,6 +1371,7 @@ async fn test_row_group_with_null_values() {
         .with_query("SELECT * FROM t WHERE \"i8\" is Null")
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(2))
+        .with_pruned_files(Some(0))
         .with_pruned_by_stats(Some(1))
         .with_expected_rows(10)
         .with_matched_by_bloom_filter(Some(0))
@@ -1303,6 +1385,7 @@ async fn test_row_group_with_null_values() {
         .with_query("SELECT * FROM t WHERE \"i16\" is Not Null")
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_pruned_by_stats(Some(2))
         .with_expected_rows(5)
         .with_matched_by_bloom_filter(Some(0))
@@ -1316,7 +1399,8 @@ async fn test_row_group_with_null_values() {
         .with_query("SELECT * FROM t WHERE \"i32\" > 7")
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(0))
-        .with_pruned_by_stats(Some(3))
+        .with_pruned_by_stats(Some(0))
+        .with_pruned_files(Some(1))
         .with_expected_rows(0)
         .with_matched_by_bloom_filter(Some(0))
         .with_pruned_by_bloom_filter(Some(0))
@@ -1332,6 +1416,7 @@ async fn test_bloom_filter_utf8_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(1)
         .with_pruned_by_bloom_filter(Some(0))
         .with_matched_by_bloom_filter(Some(1))
@@ -1344,6 +1429,7 @@ async fn test_bloom_filter_utf8_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(0)
         .with_pruned_by_bloom_filter(Some(1))
         .with_matched_by_bloom_filter(Some(0))
@@ -1356,6 +1442,7 @@ async fn test_bloom_filter_utf8_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(1)
         .with_pruned_by_bloom_filter(Some(0))
         .with_matched_by_bloom_filter(Some(1))
@@ -1368,6 +1455,7 @@ async fn test_bloom_filter_utf8_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(0)
         .with_pruned_by_bloom_filter(Some(1))
         .with_matched_by_bloom_filter(Some(0))
@@ -1383,6 +1471,7 @@ async fn test_bloom_filter_integer_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(1)
         .with_pruned_by_bloom_filter(Some(0))
         .with_matched_by_bloom_filter(Some(1))
@@ -1395,6 +1484,7 @@ async fn test_bloom_filter_integer_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(0)
         .with_pruned_by_bloom_filter(Some(1))
         .with_matched_by_bloom_filter(Some(0))
@@ -1407,6 +1497,7 @@ async fn test_bloom_filter_integer_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(1)
         .with_pruned_by_bloom_filter(Some(0))
         .with_matched_by_bloom_filter(Some(1))
@@ -1419,6 +1510,7 @@ async fn test_bloom_filter_integer_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(0)
         .with_pruned_by_bloom_filter(Some(1))
         .with_matched_by_bloom_filter(Some(0))
@@ -1434,6 +1526,7 @@ async fn test_bloom_filter_unsigned_integer_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(1)
         .with_pruned_by_bloom_filter(Some(0))
         .with_matched_by_bloom_filter(Some(1))
@@ -1446,6 +1539,7 @@ async fn test_bloom_filter_unsigned_integer_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(0)
         .with_pruned_by_bloom_filter(Some(1))
         .with_matched_by_bloom_filter(Some(0))
@@ -1461,6 +1555,7 @@ async fn test_bloom_filter_binary_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(1)
         .with_pruned_by_bloom_filter(Some(0))
         .with_matched_by_bloom_filter(Some(1))
@@ -1473,6 +1568,7 @@ async fn test_bloom_filter_binary_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(0)
         .with_pruned_by_bloom_filter(Some(1))
         .with_matched_by_bloom_filter(Some(0))
@@ -1485,6 +1581,7 @@ async fn test_bloom_filter_binary_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(1)
         .with_pruned_by_bloom_filter(Some(0))
         .with_matched_by_bloom_filter(Some(1))
@@ -1499,6 +1596,7 @@ async fn test_bloom_filter_binary_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(0)
         .with_pruned_by_bloom_filter(Some(1))
         .with_matched_by_bloom_filter(Some(0))
@@ -1514,6 +1612,7 @@ async fn test_bloom_filter_decimal_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(1)
         .with_pruned_by_bloom_filter(Some(0))
         .with_matched_by_bloom_filter(Some(1))
@@ -1526,6 +1625,7 @@ async fn test_bloom_filter_decimal_dict() {
         .with_expected_errors(Some(0))
         .with_matched_by_stats(Some(1))
         .with_pruned_by_stats(Some(1))
+        .with_pruned_files(Some(0))
         .with_expected_rows(0)
         .with_pruned_by_bloom_filter(Some(1))
         .with_matched_by_bloom_filter(Some(0))
