@@ -82,10 +82,64 @@ pub enum Operator {
     BitwiseShiftLeft,
     /// String concat
     StringConcat,
-    /// At arrow, like `@>`
+    /// At arrow, like `@>`.
+    ///
+    /// Currently only supported to be used with lists:
+    /// ```sql
+    /// select [1,3] <@ [1,2,3]
+    /// ```
     AtArrow,
-    /// Arrow at, like `<@`
+    /// Arrow at, like `<@`.
+    ///
+    /// Currently only supported to be used with lists:
+    /// ```sql
+    /// select [1,2,3] @> [1,3]
+    /// ```
     ArrowAt,
+    /// Arrow, like `->`.
+    ///
+    /// Not implemented in DataFusion yet.
+    Arrow,
+    /// Long arrow, like `->>`
+    ///
+    /// Not implemented in DataFusion yet.
+    LongArrow,
+    /// Hash arrow, like `#>`
+    ///
+    /// Not implemented in DataFusion yet.
+    HashArrow,
+    /// Hash long arrow, like `#>>`
+    ///
+    /// Not implemented in DataFusion yet.
+    HashLongArrow,
+    /// At at, like `@@`
+    ///
+    /// Not implemented in DataFusion yet.
+    AtAt,
+    /// Integer division operator, like `DIV` from MySQL or `//` from DuckDB
+    ///
+    /// Not implemented in DataFusion yet.
+    IntegerDivide,
+    /// Hash Minis, like `#-`
+    ///
+    /// Not implemented in DataFusion yet.
+    HashMinus,
+    /// At question, like `@?`
+    ///
+    /// Not implemented in DataFusion yet.
+    AtQuestion,
+    /// Question, like `?`
+    ///
+    /// Not implemented in DataFusion yet.
+    Question,
+    /// Question and, like `?&`
+    ///
+    /// Not implemented in DataFusion yet.
+    QuestionAnd,
+    /// Question pipe, like `?|`
+    ///
+    /// Not implemented in DataFusion yet.
+    QuestionPipe,
 }
 
 impl Operator {
@@ -123,7 +177,18 @@ impl Operator {
             | Operator::BitwiseShiftLeft
             | Operator::StringConcat
             | Operator::AtArrow
-            | Operator::ArrowAt => None,
+            | Operator::ArrowAt
+            | Operator::Arrow
+            | Operator::LongArrow
+            | Operator::HashArrow
+            | Operator::HashLongArrow
+            | Operator::AtAt
+            | Operator::IntegerDivide
+            | Operator::HashMinus
+            | Operator::AtQuestion
+            | Operator::Question
+            | Operator::QuestionAnd
+            | Operator::QuestionPipe => None,
         }
     }
 
@@ -216,7 +281,18 @@ impl Operator {
             | Operator::BitwiseXor
             | Operator::BitwiseShiftRight
             | Operator::BitwiseShiftLeft
-            | Operator::StringConcat => None,
+            | Operator::StringConcat
+            | Operator::Arrow
+            | Operator::LongArrow
+            | Operator::HashArrow
+            | Operator::HashLongArrow
+            | Operator::AtAt
+            | Operator::IntegerDivide
+            | Operator::HashMinus
+            | Operator::AtQuestion
+            | Operator::Question
+            | Operator::QuestionAnd
+            | Operator::QuestionPipe => None,
         }
     }
 
@@ -245,7 +321,18 @@ impl Operator {
             | Operator::BitwiseXor
             | Operator::StringConcat
             | Operator::AtArrow
-            | Operator::ArrowAt => 30,
+            | Operator::ArrowAt
+            | Operator::Arrow
+            | Operator::LongArrow
+            | Operator::HashArrow
+            | Operator::HashLongArrow
+            | Operator::AtAt
+            | Operator::IntegerDivide
+            | Operator::HashMinus
+            | Operator::AtQuestion
+            | Operator::Question
+            | Operator::QuestionAnd
+            | Operator::QuestionPipe => 30,
             Operator::Plus | Operator::Minus => 40,
             Operator::Multiply | Operator::Divide | Operator::Modulo => 45,
         }
@@ -286,6 +373,17 @@ impl fmt::Display for Operator {
             Operator::StringConcat => "||",
             Operator::AtArrow => "@>",
             Operator::ArrowAt => "<@",
+            Operator::Arrow => "->",
+            Operator::LongArrow => "->>",
+            Operator::HashArrow => "#>",
+            Operator::HashLongArrow => "#>>",
+            Operator::AtAt => "@@",
+            Operator::IntegerDivide => "DIV",
+            Operator::HashMinus => "#-",
+            Operator::AtQuestion => "@?",
+            Operator::Question => "?",
+            Operator::QuestionAnd => "?&",
+            Operator::QuestionPipe => "?|",
         };
         write!(f, "{display}")
     }
