@@ -17,6 +17,7 @@
 
 use std::any::Any;
 use std::fmt::{Debug, Formatter};
+use std::hash::{DefaultHasher, Hash, Hasher};
 use std::mem::size_of_val;
 use std::sync::Arc;
 
@@ -186,6 +187,30 @@ impl AggregateUDFImpl for ApproxPercentileContWithWeight {
     fn documentation(&self) -> Option<&Documentation> {
         self.doc()
     }
+
+    fn equals(&self, other: &dyn AggregateUDFImpl) -> bool {
+        let Some(other) = other.as_any().downcast_ref::<Self>() else {
+            return false;
+        };
+        let Self {
+            signature,
+            approx_percentile_cont,
+        } = self;
+        signature == &other.signature
+            && approx_percentile_cont.equals(&other.approx_percentile_cont)
+    }
+
+    fn hash_value(&self) -> u64 {
+        let Self {
+            signature,
+            approx_percentile_cont,
+        } = self;
+        let mut hasher = DefaultHasher::new();
+        std::any::type_name::<Self>().hash(&mut hasher);
+        signature.hash(&mut hasher);
+        hasher.write_u64(approx_percentile_cont.hash_value());
+        hasher.finish()
+    }
 }
 
 #[derive(Debug)]
