@@ -179,10 +179,6 @@ impl AggregateUDFImpl for BoolAnd {
         }
     }
 
-    fn aliases(&self) -> &[String] {
-        &[]
-    }
-
     fn order_sensitivity(&self) -> AggregateOrderSensitivity {
         AggregateOrderSensitivity::Insensitive
     }
@@ -319,10 +315,6 @@ impl AggregateUDFImpl for BoolOr {
         }
     }
 
-    fn aliases(&self) -> &[String] {
-        &[]
-    }
-
     fn order_sensitivity(&self) -> AggregateOrderSensitivity {
         AggregateOrderSensitivity::Insensitive
     }
