@@ -20,6 +20,7 @@ extern crate criterion;
 use std::sync::Arc;
 
 use arrow::array::{Array, ArrayRef, TimestampSecondArray};
+use arrow::datatypes::Field;
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
 use datafusion_common::ScalarValue;
 use rand::rngs::ThreadRng;
@@ -31,7 +32,7 @@ use datafusion_functions::datetime::date_bin;
 fn timestamps(rng: &mut ThreadRng) -> TimestampSecondArray {
     let mut seconds = vec![];
     for _ in 0..1000 {
-        seconds.push(rng.gen_range(0..1_000_000));
+        seconds.push(rng.random_range(0..1_000_000));
     }
 
     TimestampSecondArray::from(seconds)
@@ -39,7 +40,7 @@ fn timestamps(rng: &mut ThreadRng) -> TimestampSecondArray {
 
 fn criterion_benchmark(c: &mut Criterion) {
     c.bench_function("date_bin_1000", |b| {
-        let mut rng = rand::thread_rng();
+        let mut rng = rand::rng();
         let timestamps_array = Arc::new(timestamps(&mut rng)) as ArrayRef;
         let batch_len = timestamps_array.len();
         let interval = ColumnarValue::Scalar(ScalarValue::new_interval_dt(0, 1_000_000));
@@ -48,13 +49,19 @@ fn criterion_benchmark(c: &mut Criterion) {
         let return_type = udf
             .return_type(&[interval.data_type(), timestamps.data_type()])
             .unwrap();
+        let return_field = Arc::new(Field::new("f", return_type, true));
 
+        let arg_fields = vec![
+            Field::new("a", interval.data_type(), true).into(),
+            Field::new("b", timestamps.data_type(), true).into(),
+        ];
         b.iter(|| {
             black_box(
                 udf.invoke_with_args(ScalarFunctionArgs {
                     args: vec![interval.clone(), timestamps.clone()],
+                    arg_fields: arg_fields.clone(),
                     number_rows: batch_len,
-                    return_type: &return_type,
+                    return_field: Arc::clone(&return_field),
                 })
                 .expect("date_bin should work on valid values"),
             )
