@@ -17,14 +17,16 @@
 
 extern crate criterion;
 
-use arrow::datatypes::DataType;
+use arrow::datatypes::{DataType, Field};
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
 use datafusion_expr::{ScalarFunctionArgs, ScalarUDFImpl};
 use datafusion_functions::math::random::RandomFunc;
+use std::sync::Arc;
 
 fn criterion_benchmark(c: &mut Criterion) {
     let random_func = RandomFunc::new();
 
+    let return_field = Field::new("f", DataType::Float64, true).into();
     // Benchmark to evaluate 1M rows in batch size 8192
     let iterations = 1_000_000 / 8192; // Calculate how many iterations are needed to reach approximately 1M rows
     c.bench_function("random_1M_rows_batch_8192", |b| {
@@ -34,8 +36,9 @@ fn criterion_benchmark(c: &mut Criterion) {
                     random_func
                         .invoke_with_args(ScalarFunctionArgs {
                             args: vec![],
+                            arg_fields: vec![],
                             number_rows: 8192,
-                            return_type: &DataType::Float64,
+                            return_field: Arc::clone(&return_field),
                         })
                         .unwrap(),
                 );
@@ -43,6 +46,7 @@ fn criterion_benchmark(c: &mut Criterion) {
         })
     });
 
+    let return_field = Field::new("f", DataType::Float64, true).into();
     // Benchmark to evaluate 1M rows in batch size 128
     let iterations_128 = 1_000_000 / 128; // Calculate how many iterations are needed to reach approximately 1M rows with batch size 128
     c.bench_function("random_1M_rows_batch_128", |b| {
@@ -52,8 +56,9 @@ fn criterion_benchmark(c: &mut Criterion) {
                     random_func
                         .invoke_with_args(ScalarFunctionArgs {
                             args: vec![],
+                            arg_fields: vec![],
                             number_rows: 128,
-                            return_type: &DataType::Float64,
+                            return_field: Arc::clone(&return_field),
                         })
                         .unwrap(),
                 );
