@@ -22,7 +22,7 @@ use std::sync::Arc;
 use arrow::array::builder::StringBuilder;
 use arrow::array::{Array, ArrayRef, StringArray};
 use arrow::compute::cast;
-use arrow::datatypes::{DataType, TimeUnit};
+use arrow::datatypes::{DataType, Field, TimeUnit};
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
 
 use datafusion_expr::{ColumnarValue, ScalarFunctionArgs};
@@ -109,7 +109,10 @@ fn data_with_formats() -> (StringArray, StringArray, StringArray, StringArray) {
     )
 }
 fn criterion_benchmark(c: &mut Criterion) {
-    let return_type = &DataType::Timestamp(TimeUnit::Nanosecond, None);
+    let return_field =
+        Field::new("f", DataType::Timestamp(TimeUnit::Nanosecond, None), true).into();
+    let arg_field = Field::new("a", DataType::Utf8, false).into();
+    let arg_fields = vec![arg_field];
     c.bench_function("to_timestamp_no_formats_utf8", |b| {
         let arr_data = data();
         let batch_len = arr_data.len();
@@ -120,8 +123,9 @@ fn criterion_benchmark(c: &mut Criterion) {
                 to_timestamp()
                     .invoke_with_args(ScalarFunctionArgs {
                         args: vec![string_array.clone()],
+                        arg_fields: arg_fields.clone(),
                         number_rows: batch_len,
-                        return_type,
+                        return_field: Arc::clone(&return_field),
                     })
                     .expect("to_timestamp should work on valid values"),
             )
@@ -138,8 +142,9 @@ fn criterion_benchmark(c: &mut Criterion) {
                 to_timestamp()
                     .invoke_with_args(ScalarFunctionArgs {
                         args: vec![string_array.clone()],
+                        arg_fields: arg_fields.clone(),
                         number_rows: batch_len,
-                        return_type,
+                        return_field: Arc::clone(&return_field),
                     })
                     .expect("to_timestamp should work on valid values"),
             )
@@ -156,8 +161,9 @@ fn criterion_benchmark(c: &mut Criterion) {
                 to_timestamp()
                     .invoke_with_args(ScalarFunctionArgs {
                         args: vec![string_array.clone()],
+                        arg_fields: arg_fields.clone(),
                         number_rows: batch_len,
-                        return_type,
+                        return_field: Arc::clone(&return_field),
                     })
                     .expect("to_timestamp should work on valid values"),
             )
@@ -174,13 +180,22 @@ fn criterion_benchmark(c: &mut Criterion) {
             ColumnarValue::Array(Arc::new(format2) as ArrayRef),
             ColumnarValue::Array(Arc::new(format3) as ArrayRef),
         ];
+        let arg_fields = args
+            .iter()
+            .enumerate()
+            .map(|(idx, arg)| {
+                Field::new(format!("arg_{idx}"), arg.data_type(), true).into()
+            })
+            .collect::<Vec<_>>();
+
         b.iter(|| {
             black_box(
                 to_timestamp()
                     .invoke_with_args(ScalarFunctionArgs {
                         args: args.clone(),
+                        arg_fields: arg_fields.clone(),
                         number_rows: batch_len,
-                        return_type,
+                        return_field: Arc::clone(&return_field),
                     })
                     .expect("to_timestamp should work on valid values"),
             )
@@ -205,13 +220,22 @@ fn criterion_benchmark(c: &mut Criterion) {
                 Arc::new(cast(&format3, &DataType::LargeUtf8).unwrap()) as ArrayRef
             ),
         ];
+        let arg_fields = args
+            .iter()
+            .enumerate()
+            .map(|(idx, arg)| {
+                Field::new(format!("arg_{idx}"), arg.data_type(), true).into()
+            })
+            .collect::<Vec<_>>();
+
         b.iter(|| {
             black_box(
                 to_timestamp()
                     .invoke_with_args(ScalarFunctionArgs {
                         args: args.clone(),
+                        arg_fields: arg_fields.clone(),
                         number_rows: batch_len,
-                        return_type,
+                        return_field: Arc::clone(&return_field),
                     })
                     .expect("to_timestamp should work on valid values"),
             )
@@ -237,13 +261,22 @@ fn criterion_benchmark(c: &mut Criterion) {
                 Arc::new(cast(&format3, &DataType::Utf8View).unwrap()) as ArrayRef
             ),
         ];
+        let arg_fields = args
+            .iter()
+            .enumerate()
+            .map(|(idx, arg)| {
+                Field::new(format!("arg_{idx}"), arg.data_type(), true).into()
+            })
+            .collect::<Vec<_>>();
+
         b.iter(|| {
             black_box(
                 to_timestamp()
                     .invoke_with_args(ScalarFunctionArgs {
                         args: args.clone(),
+                        arg_fields: arg_fields.clone(),
                         number_rows: batch_len,
-                        return_type,
+                        return_field: Arc::clone(&return_field),
                     })
                     .expect("to_timestamp should work on valid values"),
             )
