@@ -19,13 +19,42 @@ use arrow::array::Array;
 use arrow::compute::is_not_null;
 use arrow::compute::kernels::zip::zip;
 use arrow::datatypes::DataType;
-use datafusion_common::{internal_err, Result};
-use datafusion_expr::scalar_doc_sections::DOC_SECTION_CONDITIONAL;
+use datafusion_common::{utils::take_function_args, Result};
 use datafusion_expr::{
-    ColumnarValue, Documentation, ScalarUDFImpl, Signature, Volatility,
+    ColumnarValue, Documentation, ScalarFunctionArgs, ScalarUDFImpl, Signature,
+    Volatility,
 };
-use std::sync::{Arc, OnceLock};
+use datafusion_macros::user_doc;
+use std::sync::Arc;
 
+#[user_doc(
+    doc_section(label = "Conditional Functions"),
+    description = "Returns _expression2_ if _expression1_ is NULL otherwise it returns _expression1_.",
+    syntax_example = "nvl(expression1, expression2)",
+    sql_example = r#"```sql
+> select nvl(null, 'a');
++---------------------+
+| nvl(NULL,Utf8("a")) |
++---------------------+
+| a                   |
++---------------------+\
+> select nvl('b', 'a');
++--------------------------+
+| nvl(Utf8("b"),Utf8("a")) |
++--------------------------+
+| b                        |
++--------------------------+
+```
+"#,
+    argument(
+        name = "expression1",
+        description = "Expression to return if not null. Can be a constant, column, or function, and any combination of operators."
+    ),
+    argument(
+        name = "expression2",
+        description = "Expression to return if expr1 is null. Can be a constant, column, or function, and any combination of operators."
+    )
+)]
 #[derive(Debug)]
 pub struct NVLFunc {
     signature: Signature,
@@ -88,12 +117,8 @@ impl ScalarUDFImpl for NVLFunc {
         Ok(arg_types[0].clone())
     }
 
-    fn invoke_batch(
-        &self,
-        args: &[ColumnarValue],
-        _number_rows: usize,
-    ) -> Result<ColumnarValue> {
-        nvl_func(args)
+    fn invoke_with_args(&self, args: ScalarFunctionArgs) -> Result<ColumnarValue> {
+        nvl_func(&args.args)
     }
 
     fn aliases(&self) -> &[String] {
@@ -101,53 +126,13 @@ impl ScalarUDFImpl for NVLFunc {
     }
 
     fn documentation(&self) -> Option<&Documentation> {
-        Some(get_nvl_doc())
+        self.doc()
     }
 }
 
-static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
-
-fn get_nvl_doc() -> &'static Documentation {
-    DOCUMENTATION.get_or_init(|| {
-        Documentation::builder(
-            DOC_SECTION_CONDITIONAL,
-            "Returns _expression2_ if _expression1_ is NULL otherwise it returns _expression1_.",
-            "nvl(expression1, expression2)")
-            .with_sql_example(r#"```sql
-> select nvl(null, 'a');
-+---------------------+
-| nvl(NULL,Utf8("a")) |
-+---------------------+
-| a                   |
-+---------------------+\
-> select nvl('b', 'a');
-+--------------------------+
-| nvl(Utf8("b"),Utf8("a")) |
-+--------------------------+
-| b                        |
-+--------------------------+
-```
-"#)
-            .with_argument(
-                "expression1",
-                "Expression to return if not null. Can be a constant, column, or function, and any combination of operators."
-            )
-            .with_argument(
-                "expression2",
-                "Expression to return if expr1 is null. Can be a constant, column, or function, and any combination of operators."
-            )
-            .build()
-    })
-}
-
 fn nvl_func(args: &[ColumnarValue]) -> Result<ColumnarValue> {
-    if args.len() != 2 {
-        return internal_err!(
-            "{:?} args were supplied but NVL/IFNULL takes exactly two args",
-            args.len()
-        );
-    }
-    let (lhs_array, rhs_array) = match (&args[0], &args[1]) {
+    let [lhs, rhs] = take_function_args("nvl/ifnull", args)?;
+    let (lhs_array, rhs_array) = match (lhs, rhs) {
         (ColumnarValue::Array(lhs), ColumnarValue::Scalar(rhs)) => {
             (Arc::clone(lhs), rhs.to_array_of_size(lhs.len())?)
         }
