@@ -19,13 +19,41 @@ use super::to_timestamp::ToTimestampSecondsFunc;
 use crate::datetime::common::*;
 use arrow::datatypes::{DataType, TimeUnit};
 use datafusion_common::{exec_err, Result};
-use datafusion_expr::scalar_doc_sections::DOC_SECTION_DATETIME;
 use datafusion_expr::{
     ColumnarValue, Documentation, ScalarUDFImpl, Signature, Volatility,
 };
+use datafusion_macros::user_doc;
 use std::any::Any;
-use std::sync::OnceLock;
 
+#[user_doc(
+    doc_section(label = "Time and Date Functions"),
+    description = "Converts a value to seconds since the unix epoch (`1970-01-01T00:00:00Z`). Supports strings, dates, timestamps and double types as input. Strings are parsed as RFC3339 (e.g. '2023-07-20T05:44:00') if no [Chrono formats](https://docs.rs/chrono/latest/chrono/format/strftime/index.html) are provided.",
+    syntax_example = "to_unixtime(expression[, ..., format_n])",
+    sql_example = r#"
+```sql
+> select to_unixtime('2020-09-08T12:00:00+00:00');
++------------------------------------------------+
+| to_unixtime(Utf8("2020-09-08T12:00:00+00:00")) |
++------------------------------------------------+
+| 1599566400                                     |
++------------------------------------------------+
+> select to_unixtime('01-14-2023 01:01:30+05:30', '%q', '%d-%m-%Y %H/%M/%S', '%+', '%m-%d-%Y %H:%M:%S%#z');
++-----------------------------------------------------------------------------------------------------------------------------+
+| to_unixtime(Utf8("01-14-2023 01:01:30+05:30"),Utf8("%q"),Utf8("%d-%m-%Y %H/%M/%S"),Utf8("%+"),Utf8("%m-%d-%Y %H:%M:%S%#z")) |
++-----------------------------------------------------------------------------------------------------------------------------+
+| 1673638290                                                                                                                  |
++-----------------------------------------------------------------------------------------------------------------------------+
+```
+"#,
+    argument(
+        name = "expression",
+        description = "Expression to operate on. Can be a constant, column, or function, and any combination of arithmetic operators."
+    ),
+    argument(
+        name = "format_n",
+        description = "Optional [Chrono format](https://docs.rs/chrono/latest/chrono/format/strftime/index.html) strings to use to parse the expression. Formats will be tried in the order they appear with the first successful one being returned. If none of the formats successfully parse the expression an error will be returned."
+    )
+)]
 #[derive(Debug)]
 pub struct ToUnixtimeFunc {
     signature: Signature,
@@ -62,71 +90,40 @@ impl ScalarUDFImpl for ToUnixtimeFunc {
         Ok(DataType::Int64)
     }
 
-    fn invoke_batch(
+    fn invoke_with_args(
         &self,
-        args: &[ColumnarValue],
-        batch_size: usize,
+        args: datafusion_expr::ScalarFunctionArgs,
     ) -> Result<ColumnarValue> {
-        if args.is_empty() {
+        let arg_args = &args.args;
+        if arg_args.is_empty() {
             return exec_err!("to_unixtime function requires 1 or more arguments, got 0");
         }
 
         // validate that any args after the first one are Utf8
-        if args.len() > 1 {
-            validate_data_types(args, "to_unixtime")?;
+        if arg_args.len() > 1 {
+            validate_data_types(arg_args, "to_unixtime")?;
         }
 
-        match args[0].data_type() {
+        match arg_args[0].data_type() {
             DataType::Int32 | DataType::Int64 | DataType::Null | DataType::Float64 => {
-                args[0].cast_to(&DataType::Int64, None)
+                arg_args[0].cast_to(&DataType::Int64, None)
             }
-            DataType::Date64 | DataType::Date32 | DataType::Timestamp(_, None) => args[0]
+            DataType::Date64 | DataType::Date32 => arg_args[0]
                 .cast_to(&DataType::Timestamp(TimeUnit::Second, None), None)?
                 .cast_to(&DataType::Int64, None),
-            #[allow(deprecated)] // TODO: migrate to invoke_with_args
+            DataType::Timestamp(_, tz) => arg_args[0]
+                .cast_to(&DataType::Timestamp(TimeUnit::Second, tz), None)?
+                .cast_to(&DataType::Int64, None),
             DataType::Utf8 => ToTimestampSecondsFunc::new()
-                .invoke_batch(args, batch_size)?
+                .invoke_with_args(args)?
                 .cast_to(&DataType::Int64, None),
             other => {
                 exec_err!("Unsupported data type {:?} for function to_unixtime", other)
             }
         }
     }
+
     fn documentation(&self) -> Option<&Documentation> {
-        Some(get_to_unixtime_doc())
+        self.doc()
     }
 }
-
-static DOCUMENTATION: OnceLock<Documentation> = OnceLock::new();
-
-fn get_to_unixtime_doc() -> &'static Documentation {
-    DOCUMENTATION.get_or_init(|| {
-        Documentation::builder(
-            DOC_SECTION_DATETIME,
-            "Converts a value to seconds since the unix epoch (`1970-01-01T00:00:00Z`). Supports strings, dates, timestamps and double types as input. Strings are parsed as RFC3339 (e.g. '2023-07-20T05:44:00') if no [Chrono formats](https://docs.rs/chrono/latest/chrono/format/strftime/index.html) are provided.",
-            "to_unixtime(expression[, ..., format_n])")
-            .with_argument(
-                "expression",
-                "Expression to operate on. Can be a constant, column, or function, and any combination of arithmetic operators."
-            ).with_argument(
-            "format_n",
-            "Optional [Chrono format](https://docs.rs/chrono/latest/chrono/format/strftime/index.html) strings to use to parse the expression. Formats will be tried in the order they appear with the first successful one being returned. If none of the formats successfully parse the expression an error will be returned.")
-            .with_sql_example(r#"
-```sql
-> select to_unixtime('2020-09-08T12:00:00+00:00');
-+------------------------------------------------+
-| to_unixtime(Utf8("2020-09-08T12:00:00+00:00")) |
-+------------------------------------------------+
-| 1599566400                                     |
-+------------------------------------------------+
-> select to_unixtime('01-14-2023 01:01:30+05:30', '%q', '%d-%m-%Y %H/%M/%S', '%+', '%m-%d-%Y %H:%M:%S%#z');
-+-----------------------------------------------------------------------------------------------------------------------------+
-| to_unixtime(Utf8("01-14-2023 01:01:30+05:30"),Utf8("%q"),Utf8("%d-%m-%Y %H/%M/%S"),Utf8("%+"),Utf8("%m-%d-%Y %H:%M:%S%#z")) |
-+-----------------------------------------------------------------------------------------------------------------------------+
-| 1673638290                                                                                                                  |
-+-----------------------------------------------------------------------------------------------------------------------------+
-```
-"#)
-            .build()
-    })
-}
