@@ -105,7 +105,8 @@ impl PartialSortExec {
     ) -> Self {
         debug_assert!(common_prefix_length > 0);
         let preserve_partitioning = false;
-        let cache = Self::compute_properties(&input, expr.clone(), preserve_partitioning);
+        let cache = Self::compute_properties(&input, expr.clone(), preserve_partitioning)
+            .unwrap();
         Self {
             input,
             expr,
@@ -159,7 +160,7 @@ impl PartialSortExec {
 
     /// Sort expressions
     pub fn expr(&self) -> &LexOrdering {
-        self.expr.as_ref()
+        &self.expr
     }
 
     /// If `Some(fetch)`, limits output to only the first "fetch" items
@@ -189,24 +190,22 @@ impl PartialSortExec {
         input: &Arc<dyn ExecutionPlan>,
         sort_exprs: LexOrdering,
         preserve_partitioning: bool,
-    ) -> PlanProperties {
+    ) -> Result<PlanProperties> {
         // Calculate equivalence properties; i.e. reset the ordering equivalence
         // class with the new ordering:
-        let eq_properties = input
-            .equivalence_properties()
-            .clone()
-            .with_reorder(sort_exprs);
+        let mut eq_properties = input.equivalence_properties().clone();
+        eq_properties.reorder(sort_exprs)?;
 
         // Get output partitioning:
         let output_partitioning =
             Self::output_partitioning_helper(input, preserve_partitioning);
 
-        PlanProperties::new(
+        Ok(PlanProperties::new(
             eq_properties,
             output_partitioning,
             input.pipeline_behavior(),
             input.boundedness(),
-        )
+        ))
     }
 }
 
@@ -421,7 +420,7 @@ impl PartialSortStream {
     fn sort_in_mem_batches(self: &mut Pin<&mut Self>) -> Result<RecordBatch> {
         let input_batch = concat_batches(&self.schema(), &self.in_mem_batches)?;
         self.in_mem_batches.clear();
-        let result = sort_batch(&input_batch, self.expr.as_ref(), self.fetch)?;
+        let result = sort_batch(&input_batch, &self.expr, self.fetch)?;
         if let Some(remaining_fetch) = self.fetch {
             // remaining_fetch - result.num_rows() is always be >= 0
             // because result length of sort_batch with limit cannot be
@@ -504,7 +503,7 @@ mod tests {
         };
 
         let partial_sort_exec = Arc::new(PartialSortExec::new(
-            LexOrdering::new(vec![
+            [
                 PhysicalSortExpr {
                     expr: col("a", &schema)?,
                     options: option_asc,
@@ -517,10 +516,11 @@ mod tests {
                     expr: col("c", &schema)?,
                     options: option_asc,
                 },
-            ]),
+            ]
+            .into(),
             Arc::clone(&source),
             2,
-        )) as Arc<dyn ExecutionPlan>;
+        ));
 
         let result = collect(partial_sort_exec, Arc::clone(&task_ctx)).await?;
 
@@ -569,7 +569,7 @@ mod tests {
         for common_prefix_length in [1, 2] {
             let partial_sort_exec = Arc::new(
                 PartialSortExec::new(
-                    LexOrdering::new(vec![
+                    [
                         PhysicalSortExpr {
                             expr: col("a", &schema)?,
                             options: option_asc,
@@ -582,12 +582,13 @@ mod tests {
                             expr: col("c", &schema)?,
                             options: option_asc,
                         },
-                    ]),
+                    ]
+                    .into(),
                     Arc::clone(&source),
                     common_prefix_length,
                 )
                 .with_fetch(Some(4)),
-            ) as Arc<dyn ExecutionPlan>;
+            );
 
             let result = collect(partial_sort_exec, Arc::clone(&task_ctx)).await?;
 
@@ -642,7 +643,7 @@ mod tests {
             [(1, &source_tables[0]), (2, &source_tables[1])]
         {
             let partial_sort_exec = Arc::new(PartialSortExec::new(
-                LexOrdering::new(vec![
+                [
                     PhysicalSortExpr {
                         expr: col("a", &schema)?,
                         options: option_asc,
@@ -655,7 +656,8 @@ mod tests {
                         expr: col("c", &schema)?,
                         options: option_asc,
                     },
-                ]),
+                ]
+                .into(),
                 Arc::clone(source),
                 common_prefix_length,
             ));
@@ -731,8 +733,8 @@ mod tests {
             nulls_first: false,
         };
         let schema = mem_exec.schema();
-        let partial_sort_executor = PartialSortExec::new(
-            LexOrdering::new(vec![
+        let partial_sort_exec = PartialSortExec::new(
+            [
                 PhysicalSortExpr {
                     expr: col("a", &schema)?,
                     options: option_asc,
@@ -745,17 +747,16 @@ mod tests {
                     expr: col("c", &schema)?,
                     options: option_asc,
                 },
-            ]),
+            ]
+            .into(),
             Arc::clone(&mem_exec),
             1,
         );
-        let partial_sort_exec =
-            Arc::new(partial_sort_executor.clone()) as Arc<dyn ExecutionPlan>;
         let sort_exec = Arc::new(SortExec::new(
-            partial_sort_executor.expr,
-            partial_sort_executor.input,
-        )) as Arc<dyn ExecutionPlan>;
-        let result = collect(partial_sort_exec, Arc::clone(&task_ctx)).await?;
+            partial_sort_exec.expr.clone(),
+            Arc::clone(&partial_sort_exec.input),
+        ));
+        let result = collect(Arc::new(partial_sort_exec), Arc::clone(&task_ctx)).await?;
         assert_eq!(
             result.iter().map(|r| r.num_rows()).collect_vec(),
             [125, 125, 150]
@@ -792,8 +793,8 @@ mod tests {
             (Some(150), vec![125, 25]),
             (Some(250), vec![125, 125]),
         ] {
-            let partial_sort_executor = PartialSortExec::new(
-                LexOrdering::new(vec![
+            let partial_sort_exec = PartialSortExec::new(
+                [
                     PhysicalSortExpr {
                         expr: col("a", &schema)?,
                         options: option_asc,
@@ -806,19 +807,22 @@ mod tests {
                         expr: col("c", &schema)?,
                         options: option_asc,
                     },
-                ]),
+                ]
+                .into(),
                 Arc::clone(&mem_exec),
                 1,
             )
             .with_fetch(fetch_size);
 
-            let partial_sort_exec =
-                Arc::new(partial_sort_executor.clone()) as Arc<dyn ExecutionPlan>;
             let sort_exec = Arc::new(
-                SortExec::new(partial_sort_executor.expr, partial_sort_executor.input)
-                    .with_fetch(fetch_size),
-            ) as Arc<dyn ExecutionPlan>;
-            let result = collect(partial_sort_exec, Arc::clone(&task_ctx)).await?;
+                SortExec::new(
+                    partial_sort_exec.expr.clone(),
+                    Arc::clone(&partial_sort_exec.input),
+                )
+                .with_fetch(fetch_size),
+            );
+            let result =
+                collect(Arc::new(partial_sort_exec), Arc::clone(&task_ctx)).await?;
             assert_eq!(
                 result.iter().map(|r| r.num_rows()).collect_vec(),
                 expected_batch_num_rows
@@ -847,8 +851,8 @@ mod tests {
             nulls_first: false,
         };
         let fetch_size = Some(250);
-        let partial_sort_executor = PartialSortExec::new(
-            LexOrdering::new(vec![
+        let partial_sort_exec = PartialSortExec::new(
+            [
                 PhysicalSortExpr {
                     expr: col("a", &schema)?,
                     options: option_asc,
@@ -857,15 +861,14 @@ mod tests {
                     expr: col("c", &schema)?,
                     options: option_asc,
                 },
-            ]),
+            ]
+            .into(),
             Arc::clone(&mem_exec),
             1,
         )
         .with_fetch(fetch_size);
 
-        let partial_sort_exec =
-            Arc::new(partial_sort_executor.clone()) as Arc<dyn ExecutionPlan>;
-        let result = collect(partial_sort_exec, Arc::clone(&task_ctx)).await?;
+        let result = collect(Arc::new(partial_sort_exec), Arc::clone(&task_ctx)).await?;
         for rb in result {
             assert!(rb.num_rows() > 0);
         }
@@ -898,10 +901,11 @@ mod tests {
             TestMemoryExec::try_new_exec(&[vec![batch]], Arc::clone(&schema), None)?;
 
         let partial_sort_exec = Arc::new(PartialSortExec::new(
-            LexOrdering::new(vec![PhysicalSortExpr {
+            [PhysicalSortExpr {
                 expr: col("field_name", &schema)?,
                 options: SortOptions::default(),
-            }]),
+            }]
+            .into(),
             input,
             1,
         ));
@@ -987,7 +991,7 @@ mod tests {
         )?;
 
         let partial_sort_exec = Arc::new(PartialSortExec::new(
-            LexOrdering::new(vec![
+            [
                 PhysicalSortExpr {
                     expr: col("a", &schema)?,
                     options: option_asc,
@@ -1000,7 +1004,8 @@ mod tests {
                     expr: col("c", &schema)?,
                     options: option_desc,
                 },
-            ]),
+            ]
+            .into(),
             TestMemoryExec::try_new_exec(&[vec![batch]], schema, None)?,
             2,
         ));
@@ -1062,10 +1067,11 @@ mod tests {
         let blocking_exec = Arc::new(BlockingExec::new(Arc::clone(&schema), 1));
         let refs = blocking_exec.refs();
         let sort_exec = Arc::new(PartialSortExec::new(
-            LexOrdering::new(vec![PhysicalSortExpr {
+            [PhysicalSortExpr {
                 expr: col("a", &schema)?,
                 options: SortOptions::default(),
-            }]),
+            }]
+            .into(),
             blocking_exec,
             1,
         ));
