@@ -27,6 +27,7 @@ use datafusion_functions_window_common::field::WindowUDFFieldArgs;
 use datafusion_functions_window_common::partition::PartitionEvaluatorArgs;
 use std::any::Any;
 use std::fmt::Debug;
+use std::hash::{DefaultHasher, Hash, Hasher};
 
 mod roundtrip_logical_plan;
 mod roundtrip_physical_plan;
@@ -80,6 +81,34 @@ impl ScalarUDFImpl for MyRegexUdf {
     fn aliases(&self) -> &[String] {
         &self.aliases
     }
+
+    fn equals(&self, other: &dyn ScalarUDFImpl) -> bool {
+        let Some(other) = other.as_any().downcast_ref::<Self>() else {
+            return false;
+        };
+        let Self {
+            signature,
+            pattern,
+            aliases,
+        } = self;
+        signature == &other.signature
+            && pattern == &other.pattern
+            && aliases == &other.aliases
+    }
+
+    fn hash_value(&self) -> u64 {
+        let Self {
+            signature,
+            pattern,
+            aliases,
+        } = self;
+        let mut hasher = DefaultHasher::new();
+        std::any::type_name::<Self>().hash(&mut hasher);
+        signature.hash(&mut hasher);
+        pattern.hash(&mut hasher);
+        aliases.hash(&mut hasher);
+        hasher.finish()
+    }
 }
 
 #[derive(Clone, PartialEq, ::prost::Message)]
@@ -123,6 +152,23 @@ impl AggregateUDFImpl for MyAggregateUDF {
     ) -> datafusion_common::Result<Box<dyn Accumulator>> {
         unimplemented!()
     }
+
+    fn equals(&self, other: &dyn AggregateUDFImpl) -> bool {
+        let Some(other) = other.as_any().downcast_ref::<Self>() else {
+            return false;
+        };
+        let Self { signature, result } = self;
+        signature == &other.signature && result == &other.result
+    }
+
+    fn hash_value(&self) -> u64 {
+        let Self { signature, result } = self;
+        let mut hasher = DefaultHasher::new();
+        std::any::type_name::<Self>().hash(&mut hasher);
+        signature.hash(&mut hasher);
+        result.hash(&mut hasher);
+        hasher.finish()
+    }
 }
 
 #[derive(Clone, PartialEq, ::prost::Message)]
@@ -172,6 +218,23 @@ impl WindowUDFImpl for CustomUDWF {
     ) -> datafusion_common::Result<FieldRef> {
         Ok(Field::new(field_args.name(), DataType::UInt64, false).into())
     }
+
+    fn equals(&self, other: &dyn WindowUDFImpl) -> bool {
+        let Some(other) = other.as_any().downcast_ref::<Self>() else {
+            return false;
+        };
+        let Self { signature, payload } = self;
+        signature == &other.signature && payload == &other.payload
+    }
+
+    fn hash_value(&self) -> u64 {
+        let Self { signature, payload } = self;
+        let mut hasher = DefaultHasher::new();
+        std::any::type_name::<Self>().hash(&mut hasher);
+        signature.hash(&mut hasher);
+        payload.hash(&mut hasher);
+        hasher.finish()
+    }
 }
 
 #[derive(Debug)]
